"""
CLI to our fake dagman executor and fake job executor
"""

import argparse as ap
from pathlib import Path

from .condorlite import Job


def execute_dag(file: Path):
    """
    Execute the DAG in the given file.
    :param file:
    :return:
    """
    dag = Dag.parse(file)
    dag.execute()


def execute_job(file: Path):
    """
    Execute the job in the given file.
    :param file:
    :return:
    """
    job = Job(file)
    job.execute()


def main():
    parser = ap.ArgumentParser('vulture', description='Tool to run HTCondor job specs and DAG workflows locally')
    subparsers = parser.add_subparsers()
    dag = subparsers.add_parser('dag', help='DAG operations')
    sub2 = dag.add_subparsers()
    execute = sub2.add_parser('execute', description='Execute an HTCondor DAGMAN DAG')
    execute.set_defaults(func=execute_dag)
    execute.add_argument('file', nargs=1, type=Path, help='DAGMAN file to execute')

    job = subparsers.add_parser('job', help='Job operations')
    sub2 = job.add_subparsers()
    execute = sub2.add_parser('execute', description='Execute an HTCondor job specification')
    execute.set_defaults(func=execute_job)
    execute.add_argument('file', nargs=1, type=Path, help='Job file to execute')

    args = parser.parse_args()

    if 'func' in args and 'file' in args:
        for f in args.file:
            args.func(f)
    else:
        parser.print_usage()
