''' fake projects to use in testing scode_project_updater '''

import warnings

from sqlalchemy import exc as sa_exc

from schema import Author, Project


class ScodeTestProject():
    ''' fake s_code project '''
    def __init__(self):
        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)


            self.project_code = 'SCODE_TEST_PROJECT'
            self.project = Project (
                project_code = self.project_code,
                starttime = 58551.9380295139,
                endtime = 58924.9589184028,
                proprietary_duration = 365,
                title = 'Copy of SK0442',
                abstract = ' Lorem ipsum dolor sit amet, consectetur adipiscing ' \
                           'elit, sed do eiusmod tempor incididunt ut labore et ' \
                           'dolore magna aliqua. Ut enim ad minim veniam, quis ' \
                           'nostrud exercitation ullamco laboris nisi ut aliquip ' \
                           'ex ea commodo consequat. Duis aute irure dolor in ' \
                           'reprehenderit in voluptate velit esse cillum dolore ' \
                           'eu fugiat nulla pariatur. Excepteur sint occaecat ' \
                           'cupidatat non proident, sunt in culpa qui officia ' \
                           'deserunt mollit anim id est laborum.'
            )
            self.project.authors = [
                Author(project_code=self.project_code,
                       username='srandall',
                       firstname='Scott',
                       lastname='Randall',
                       pst_person_id='4686',
                       is_pi=True),

                Author(project_code=self.project_code,
                       username='s.giacintucci',
                       firstname='Simona',
                       lastname='Giacintucci',
                       pst_person_id='317',
                       is_pi=False),

                Author(project_code=self.project_code,
                       username='esch44',
                       firstname='Emma',
                       lastname='Schwartzman',
                       pst_person_id='11991',
                       is_pi=False),

                Author(project_code=self.project_code,
                       username='tclarke',
                       firstname='Tracy',
                       lastname='Clarke',
                       pst_person_id='341',
                       is_pi=False),

            ]


class ScienceTestProject():
    ''' fake VLA science project '''
    def __init__(self):
        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            self.project_code = 'VLA_TEST_PROJECT'
            self.project = Project(
                project_code = self.project_code,
                starttime=56688.8103640046,
                endtime=56810.4840497685,
                title='Copy of 13B-014 for testing',
                abstract='Spurred by our surprising VLA discovery of the first ' \
                         'black holes in Milky Way ' \
                         'globular clusters, we propose an ambitious survey for both stellar-mass and ' \
                         'intermediate-mass black holes in globular clusters. ' \
                         'With well-defined selection criteria, our sample will allow the first statistical ' \
                         'determination of the presence of black holes in clusters. This survey will make an ' \
                         'immediate impact in a number of fields, including black hole demographics, ' \
                         'accretion physics, gravitational wave predictions, and globular cluster evolution.'
            )
            self.project.authors = [
                Author(project_code=self.project_code,
                       username='jstrader',
                       firstname='Jay',
                       lastname='Strader',
                       pst_person_id='4064',
                       is_pi=True),
                Author(project_code=self.project_code,
                       username='jcamj',
                       firstname='James',
                       lastname='Miller-Jones',
                       pst_person_id='490',
                       is_pi=False),
                Author(project_code=self.project_code,
                       username='chomiuk',
                       firstname='Laura',
                       lastname='Chomiuk',
                       pst_person_id='701',
                       is_pi=False),
                Author(project_code=self.project_code,
                       username='gsivakoff',
                       firstname='Gregory',
                       lastname='Sivakoff',
                       pst_person_id='834',
                       is_pi=False),
                Author(project_code=self.project_code,
                       username='tjmaccarone',
                       firstname='Thomas',
                       lastname='Maccarone',
                       pst_person_id='887',
                       is_pi=False),

                Author(project_code=self.project_code,
                       username='anilseth',
                       firstname='Anil',
                       lastname='Setn',
                       pst_person_id='1197',
                       is_pi=False),

                Author(project_code=self.project_code,
                       username='Craig Heinke',
                       firstname='Craig',
                       lastname='Heinke',
                       pst_person_id='3729',
                       is_pi=False),

                Author(project_code=self.project_code,
                       username='evanoyola',
                       firstname='Eva',
                       lastname='Noyola',
                       pst_person_id='5532',
                       is_pi=False),

            ]

class AlmaTestProject():
    ''' fake ALMA project '''
    def __init__(self):
        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            self.project_code = 'ALMA_TEST_PROJECT'
            self.project = Project(
                project_code=self.project_code,
                starttime=56799.3877155556,
                endtime=56799.4128683333,
                proprietary_duration=365,
                title='Copy of 2012.1.00060.S for testing',
                abstract = "Understanding the physical factors that control the " \
                           "conversion of interstellar gas into stars " \
                           "is of fundamental importance for both developing a predictive physical theory of star formation and understanding the evolution of galaxies from the earliest epochs of cosmic history to the present time. An important aspect of this question is the study of empirical relations that connect the star formation rate in a given region to local properties of the interstellar medium. An important example is the Schmidt-Kennicutt (KS) law for galaxies that relates the surface densities of the star formation rate and the surface densities of interstellar gas in a non-linear fashion. However, it is also known that there is a linear correlation between the total SFR in galaxies and the mass of dense molecular gas as traced by the high excitation HCN molecule. Contrary to the KS relation, this scaling relation suggests that the total SFR depends simply on the total amount of dense molecular gas in a star forming system. Recently, we have begun to test these scaling relations in the Galactic neighborhood where star formation rates can be much better constrained. We found that for local clouds the total SFR scales most directly, and linearly, with the total mass of high extinction (and dense) molecular gas. Furthermore, we found this linear scaling law between SFR and dense gas to extend and extrapolate directly and smoothly to external galaxies. Moreover, our observations also demonstrate that a KS type relation does not exist for molecular clouds in the Galactic neighborhood. This is a direct consequence of a well known scaling law between the mass and size of molecular clouds, Larson's third law. Overall, our results indicate that a linear scaling law, in which the total amount of dense gas controls the SFR, is the fundamental physical relation that connects star formation across the vast scales from individual GMCs to entire galaxies. Critical testing of these ideas require resolved observations of GMCs in external galaxies. Here we propose to use ALMA to evaluate star formation scaling laws in a nearby galaxy where we can obtain resolved observations of individual GMCs. This allows us to obtain observations of a larger sample of GMCs than is accessible in the Galactic neighborhood. An extensive APEX survey of HII regions in the nearby galaxy NGC 300 has provided us with a sample of 36 star-forming regions with CO(2-1) detections and 42 upper limits. We are currently working on obtaining star formation rates for these regions from multi-wavelength ancillary data including our Herschel observations. We propose to use ALMA's unequalled capabilities to obtain snapshot observations of 40 selected regions in CO(2-1) in order to make resolved measurements of cloud structure to obtain sizes and virial masses. As a pilot project, we also propose to observe the brightest subsample in HCN(1-0) as a dense-gas tracer. Our proposed ALMA CO observations will enable us to to test Larson's scaling laws in an external galaxy and to evaluate which formulation of the Schmidt law is the most meaningful and appropriate to apply to spiral galaxies, and in doing so refine Schmidt's original conjecture of a scaling relation between the rate of star formation and gas density."
            )
            self.project.authors = [
                Author(project_code=self.project_code,
                       username='clada',
                       firstname='Charles',
                       lastname='Lada',
                       pst_person_id=None,
                       is_pi=True),

                Author(project_code=self.project_code,
                       username='jforbrich',
                       firstname='Jan',
                       lastname='Forbrich',
                       pst_person_id=None,
                       is_pi=False),
                Author(project_code=self.project_code,
                       username='cfaesi',
                       firstname='Christopher',
                       lastname='Faesi',
                       pst_person_id=None,
                       is_pi=False),
            ]

fake_projects = [ScodeTestProject().project,
                 ScienceTestProject().project,
                 AlmaTestProject().project]

def get_test_project(project_code: str):
    for project in fake_projects:
        if project.project_code == project_code:
            return project
    return None

def get_author_pst_ids(project: Project):
    ''' build list of pst_person_ids for display;
        we want the PI's pst_person_id followed by the CoIs' pst_person_ids
        in numeric order
    '''
    project_pi = project.authors[0]
    coi_pst_ids = [int(coi.pst_person_id) for coi in project.authors[1:]]
    coi_pst_ids = sorted(coi_pst_ids)

    author_pst_ids = [int(project_pi.pst_person_id)]
    [author_pst_ids.append(id) for id in coi_pst_ids]
    return [id for id in author_pst_ids]

